@@ -46,14 +46,14 @@ def _read_csv(config, mapping_rule, references, file_source_type):
 
     return pd.read_table(mapping_rule['data_source'],
                          delimiter=delimiter,
-                         index_col=False,   # TODO: use None?
+                         index_col=False,
                          encoding='utf-8',
                          encoding_errors='strict',
                          usecols=references,
                          chunksize=config.get_chunksize(),
                          engine='c',
                          dtype=str,
-                         memory_map=False,  # TODO: use True?
+                         memory_map=False,
                          keep_default_na=False,
                          na_values=config.get_na_values(),
                          na_filter=config.apply_na_filter())
